package rnd.agentController;

import java.util.concurrent.CountDownLatch;
import javax.jms.MessageProducer;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;

/**
 * Message composer class, used to create new message and put it into JMS Queue.
 * 
 * @author Ritesh.Ranpara
 * 
 */
public class MessageComposer implements Runnable {
	/**
	 * Composer ID.
	 */
	private String message;

	/**
	 * Receiver ID.
	 */
	private String receiverId;
	/**
	 * Done signal used for multi-threading.
	 */
	private final CountDownLatch doneSignal;

	/**
	 * Constructor.
	 * 
	 * @param strMessage
	 */
	public MessageComposer(final String strMessage, final String strRxId,
			final CountDownLatch doneSignal) {
		message = strMessage;
		receiverId = strRxId;
		this.doneSignal = doneSignal;
	}

	/**
	 * Generates message & puts it in the queue.
	 */
	@Override
	public void run() {
		try {
			QueueConnection queueConnection = QueueConfiguration
					.getQueueConnectionFactory().createQueueConnection();
			QueueSession queueSession = queueConnection.createQueueSession(
					false, Session.AUTO_ACKNOWLEDGE);
			MessageProducer queueProducer = queueSession
					.createProducer(QueueConfiguration.getQueue());

			for (int i = 0; i < 5; i++) {
				// message
				TextMessage textMessage = queueSession
						.createTextMessage(getMessage());
				// specifying receiver side property that must match to receive
				// this
				// message
				textMessage.setStringProperty(QueueConfiguration.RX_ID,
						receiverId);

				queueProducer.send(textMessage);
				System.out.println("Sent message :: \"" + textMessage.getText()
						+ "\"");
			}
			// closing the connection : this will give immediate effect on
			// JMS Server side Consumer Count value.
			queueConnection.close();
			// done signal
			doneSignal.countDown();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param message
	 *            the message to set
	 */
	public void setMessage(String message) {
		this.message = message;
	}

	/**
	 * @return the message
	 */
	public String getMessage() {
		return message;
	}

	/**
	 * @param receiverId
	 *            the receiverId to set
	 */
	public void setReceiverId(String receiverId) {
		this.receiverId = receiverId;
	}

	/**
	 * @return the receiverId
	 */
	public String getReceiverId() {
		return receiverId;
	}

	/**
	 * @return the doneSignal
	 */
	public CountDownLatch getDoneSignal() {
		return doneSignal;
	}
}
