package rnd.agentController;

import javax.jms.QueueConnection;
import javax.jms.QueueReceiver;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;

/**
 * Message receiver class used to receive message from JMS Queue.
 * 
 * @author Ritesh.Ranpara
 * 
 */
public class MessageReceiver implements Runnable {
	/**
	 * Receiver ID.
	 */
	private String receiverId;

	/**
	 * Receiver ID.
	 * 
	 * @param rxId
	 */
	public MessageReceiver(final String rxId) {
		receiverId = rxId;
	}

	/**
	 * THIS run() is not working as per expectation.
	 * It will read the message from JMS queue by matching certain property of
	 * the message.
	 */
	@Override
	public void run() {
		try {
			QueueConnection queueConnection = QueueConfiguration
					.getQueueConnectionFactory().createQueueConnection();
			QueueSession queueSession = queueConnection.createQueueSession(
					false, Session.AUTO_ACKNOWLEDGE);
			// Create receiver with certain property
			QueueReceiver queueReceiver = queueSession.createReceiver(
					QueueConfiguration.getQueue(), QueueConfiguration.RX_ID
							+ "='" + getReceiverId() + "'");
			queueConnection.start();
			// this is blocking call, this method will wait till any
			// message is received
			for (int i = 0; i < 5; i++) {

				System.out.println("Receiver " + getReceiverId()
						+ " Waiting for message:");
				TextMessage m = (TextMessage) queueReceiver.receive();
				System.out.println(getReceiverId() + " received message :: \""
						+ m.getText() + "\"");
				// closing the connection : this will give immediate effect on
				// JMS Server side Consumer Count value.
			}
			queueConnection.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * THIS run() WORKING FINE
	 * It will read the message from JMS queue by matching certain property of
	 * the message.
	*/
	/*
	@Override
	public void run() {
		try {
			QueueConnection queueConnection = QueueConfiguration
					.getQueueConnectionFactory().createQueueConnection();
			queueConnection.start();
			// this is blocking call, this method will wait till any
			// message is received
			for (int i = 0; i < 5; i++) {
				QueueSession queueSession = queueConnection.createQueueSession(
						false, Session.AUTO_ACKNOWLEDGE);
				// Create receiver with certain property
				QueueReceiver queueReceiver = queueSession.createReceiver(
						QueueConfiguration.getQueue(), QueueConfiguration.RX_ID
								+ "='" + getReceiverId() + "'");

				System.out.println("Receiver " + getReceiverId()
						+ " Waiting for message:");
				TextMessage m = (TextMessage) queueReceiver.receive();
				System.out.println(getReceiverId() + " received message :: \""
						+ m.getText() + "\"");
				// closing the connection : this will give immediate effect on
				// JMS Server side Consumer Count value.
				queueSession.close();
			}
			queueConnection.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	*/

	
	/**
	 * @param receiverId
	 *            the receiverId to set
	 */
	public void setReceiverId(String receiverId) {
		this.receiverId = receiverId;
	}

	/**
	 * @return the receiverId
	 */
	public String getReceiverId() {
		return receiverId;
	}
}
