package rnd.agentController;

import java.util.Properties;
import javax.jms.Queue;
import javax.jms.QueueConnectionFactory;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * Queue configuration details and its connectivity done from this class.
 * 
 * @author Ritesh.Ranpara
 * 
 */
public class QueueConfiguration {
	/**
	 * JMS/JMX Server
	 */
	public final static String JMS_SERVER = "jmsserver.local";
	/**
	 * Queue connection factory name.
	 */
	private final static String QUEUE_CONNECTION_FACTORY = "/ConnectionFactory";
	/**
	 * Controller Queue Name.
	 */
	public final static String QUEUE_NAME = "JobQueue";
	/**
	 * JNDI Queue name.
	 */
	private final static String JNDI_QUEUE_NAME = "/queue/" + QUEUE_NAME;
	/**
	 * JMX Url.
	 */
	public static final String JMX_URL = "service:jmx:rmi:///jndi/rmi://"
			+ JMS_SERVER + ":1090/jmxrmi";
	/**
	 * Properties for JNDI.
	 */
	private final static Properties properties = new Properties();
	/**
	 * JNDI Context.
	 */
	private static Context jndiContext;
	/**
	 * Queue Connection Factory.
	 */
	private static QueueConnectionFactory queueConnectionFactory;
	/**
	 * JMS Queue.
	 */
	private static Queue queue;

	/**
	 * System/Receiver specific property name.
	 */
	public static final String RX_ID = "AGENT";

	/**
	 * Initializing Queue.
	 */
	public static void initializeQueue() {
		// clear existing values
		properties.clear();
		// set new values
		properties.put("java.naming.factory.initial",
				"org.jnp.interfaces.NamingContextFactory");

		properties.put("java.naming.provider.url", "jnp://" + JMS_SERVER
				+ ":1099");

		properties.put("java.naming.factory.url.pkgs",
				"org.jboss.naming:org.jnp.interfaces");

		try {
			// get JNDI context
			jndiContext = new InitialContext(properties);
			queueConnectionFactory = (QueueConnectionFactory) jndiContext
					.lookup(QUEUE_CONNECTION_FACTORY);
			queue = (Queue) jndiContext.lookup(JNDI_QUEUE_NAME);
		} catch (NamingException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @return the queueConnectionFactory
	 */
	public static QueueConnectionFactory getQueueConnectionFactory() {
		return queueConnectionFactory;
	}

	/**
	 * @return the queue
	 */
	public static Queue getQueue() {
		return queue;
	}
}
