package rnd.agentController;

import java.util.Hashtable;
import java.util.Map.Entry;
import java.util.Random;
import java.util.concurrent.CountDownLatch;

/**
 * Demonstrates the use of System Controller Queue by which we can sent
 * particular message to particular receiver and can control it.
 * 
 * @author Ritesh.Ranpara
 * 
 */
public class SystemControllerQueueDemo {
	/**
	 * MessageReceiver IDs.
	 */
	public static final String[] MSG_RECEIVER_IDS = { "System1", "System2", "System3",
			"System4", "System5" };

	/**
	 * Map which contains Message as Key and Receiver-ID as Value.
	 */
	private static final Hashtable<String, String> mapMsgReceiver = new Hashtable<String, String>();

	/**
	 * Initializing block.
	 */
	static {
		// initialize queue configuration
		QueueConfiguration.initializeQueue();
		Random random = new Random();
		// set queue messages
		for (final String rxId : MSG_RECEIVER_IDS) {
			mapMsgReceiver.put("Message ID is " + random.nextInt(1000)
					+ ", This message is for " + rxId, rxId);
		}
	}

	/**
	 * Main.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		// latch to wait till all messages sent by MessageComposer
		CountDownLatch doneSignal = new CountDownLatch(mapMsgReceiver.size());

		// send messages to queue with its receiver/destination ID
		for (Entry<String, String> entry : mapMsgReceiver.entrySet()) {
			Thread thread = new Thread(new MessageComposer(entry.getKey(),
					entry.getValue(), doneSignal));
			thread.start();
		}

		// wait till all messages sent
		try {
			doneSignal.await();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		// start the receivers
		for (String rxId : MSG_RECEIVER_IDS) {
			Thread thread = new Thread(new MessageReceiver(rxId));
			thread.start();
		}
	}

	/**
	 * Check for valid receiver for given Message & Receiver ID.
	 * 
	 * @return true/false.
	 */
	public static boolean checkForValidReceiver(final String rxMsg,
			final String rxId) {
		return mapMsgReceiver.get(rxMsg).equals(rxId);
	}
}
